//
//  MainViewController.m
//  Knock
//
//  Created by Anna Wang on 11/19/15.
//  Copyright © 2015 Anna Wang. All rights reserved.
//

#import "MainViewController.h"
#import "MapViewController.h"
#import "MapMenuViewController.h"

#import <QuartzCore/QuartzCore.h>

#define CENTER_TAG 1
#define RIGHT_PANEL_TAG 3
#define CORNER_RADIUS 4
#define SLIDE_TIMING .25
#define PANEL_WIDTH 170

@interface MainViewController () <MapViewControllerDelegate, UIGestureRecognizerDelegate>

@property (nonatomic, strong) UINavigationController *centerViewController;
@property (nonatomic, strong) MapViewController *mapViewController;
@property (nonatomic, strong) MapMenuViewController *rightPanelViewController;
@property (nonatomic, assign) BOOL showingRightPanel;
@property (nonatomic, assign) BOOL showPanel;
@property (nonatomic, assign) CGPoint preVelocity;
@property (nonatomic, assign) NSUInteger distance;
@property (nonatomic, assign) BOOL friends;
@property (nonatomic, assign) BOOL strangers;
@property (nonatomic, assign) BOOL fOF;

@end

@implementation MainViewController

#pragma mark -
#pragma mark View Did Load/Unload

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.friends = YES;
    self.fOF = YES;
    self.strangers = NO;
    self.distance = 2;
    
    [self setupView];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

#pragma mark -
#pragma mark View Will/Did Appear

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

#pragma mark -
#pragma mark View Will/Did Disappear

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

#pragma mark -
#pragma mark Setup View

- (void)setupView
{
    self.centerViewController = [[self storyboard] instantiateViewControllerWithIdentifier:@"map_nav_controller"];
    self.mapViewController = [self.centerViewController.viewControllers objectAtIndex:0];
    self.mapViewController.view.tag = CENTER_TAG;
    self.mapViewController.delegate = self;
    
    [self.view addSubview:self.centerViewController.view];
    [self addChildViewController:_centerViewController];
    
    [_centerViewController didMoveToParentViewController:self];
    [self setupGestures];
}

- (void)showCenterViewWithShadow:(BOOL)value withOffset:(double)offset
{
    if (value)
    {
        [_centerViewController.view.layer setCornerRadius:CORNER_RADIUS];
        [_centerViewController.view.layer setShadowColor:[UIColor blackColor].CGColor];
        [_centerViewController.view.layer setShadowOpacity:0.8];
        [_centerViewController.view.layer setShadowOffset:CGSizeMake(offset, offset)];
        
    }
    else
    {
        [_centerViewController.view.layer setCornerRadius:0.0f];
        [_centerViewController.view.layer setShadowOffset:CGSizeMake(offset, offset)];
    }
}

- (void)resetMainView
{
    if (_rightPanelViewController != nil)
    {
        [self.rightPanelViewController.view removeFromSuperview];
        self.rightPanelViewController = nil;
        
        _mapViewController.rightButton.tag = 1;
        self.showingRightPanel = NO;
    }
    
    // remove view shadows
    [self showCenterViewWithShadow:NO withOffset:0];
}

- (UIView *)getRightView
{
    // init view if it doesn't already exist
    if (_rightPanelViewController == nil)
    {
        // this is where you define the view for the right panel
        self.rightPanelViewController = [[self storyboard] instantiateViewControllerWithIdentifier:@"map_menu_view_controller"];

        self.rightPanelViewController.view.tag = RIGHT_PANEL_TAG;
        self.rightPanelViewController.delegate = _mapViewController;
        
        [self.view addSubview:self.rightPanelViewController.view];
        
        [self addChildViewController:self.rightPanelViewController];
        [_rightPanelViewController didMoveToParentViewController:self];
        
        _rightPanelViewController.view.frame = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    }
    
    self.showingRightPanel = YES;
    
    // set up view shadows
    [self showCenterViewWithShadow:YES withOffset:2];
    
    UIView *view = self.rightPanelViewController.view;
    return view;
}

#pragma mark -
#pragma mark Swipe Gesture Setup/Actions

#pragma mark - setup

- (void)setupGestures
{
    UIPanGestureRecognizer *panRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(movePanel:)];
    [panRecognizer setMinimumNumberOfTouches:1];
    [panRecognizer setMaximumNumberOfTouches:1];
    [panRecognizer setDelegate:self];
    
    [_centerViewController.view addGestureRecognizer:panRecognizer];
}

-(void)movePanel:(id)sender
{
//    [[[(UITapGestureRecognizer*)sender view] layer] removeAllAnimations];
//    
//    CGPoint translatedPoint = [(UIPanGestureRecognizer*)sender translationInView:self.view];
//    CGPoint velocity = [(UIPanGestureRecognizer*)sender velocityInView:[sender view]];
//    
//    if([(UIPanGestureRecognizer*)sender state] == UIGestureRecognizerStateBegan) {
//        UIView *childView = nil;
//        
//        if(velocity.x < 0) {
//            childView = [self getRightView];
//            // Make sure the view you're working with is front and center.
//            [self.view sendSubviewToBack:childView];
//            [[sender view] bringSubviewToFront:[(UIPanGestureRecognizer*)sender view]];
//        }
//    }
//    
//    if([(UIPanGestureRecognizer*)sender state] == UIGestureRecognizerStateEnded) {
//        if(velocity.x > 0) {
//            // NSLog(@"gesture went right");
//        } else {
//            // NSLog(@"gesture went left");
//        }
//        
//        if (!_showPanel) {
//            [self movePanelToOriginalPosition];
//        } else {
//            if (_showingRightPanel) {
//                [self movePanelLeft];
//            }
//        }
//    }
//    
//    if([(UIPanGestureRecognizer*)sender state] == UIGestureRecognizerStateChanged) {
//        if((velocity.x > 0 && _showingRightPanel) || velocity.x < 0) {
//            // NSLog(@"gesture went right");
//            //        } else {
//            // NSLog(@"gesture went left");
//            
//            // Are you more than halfway? If so, show the panel when done dragging by setting this value to YES (1).
//            _showPanel = abs([sender view].center.x - _centerViewController.view.frame.size.width/2) > _centerViewController.view.frame.size.width/2;
//            
//            // Allow dragging only in x-coordinates by only updating the x-coordinate with translation position.
//            [sender view].center = CGPointMake([sender view].center.x + translatedPoint.x, [sender view].center.y);
//            [(UIPanGestureRecognizer*)sender setTranslation:CGPointMake(0,0) inView:self.view];
//            
//            // If you needed to check for a change in direction, you could use this code to do so.
//            if(velocity.x*_preVelocity.x + velocity.y*_preVelocity.y > 0) {
//                // NSLog(@"same direction");
//            } else {
//                // NSLog(@"opposite direction");
//            }
//            
//            _preVelocity = velocity;
//        }
//    }
}

#pragma mark -
#pragma mark Delegate Actions


- (void)movePanelLeft // to show right panel
{
    UIView *childView = [self getRightView];
    [self.view sendSubviewToBack:childView];
    [((MapMenuViewController *)self.rightPanelViewController) updateFilters:self.distance friends:self.friends fOF:self.fOF strangers:self.strangers];
    [UIView animateWithDuration:SLIDE_TIMING delay:0 options:UIViewAnimationOptionBeginFromCurrentState
                     animations:^{
                         _centerViewController.view.frame = CGRectMake(-self.view.frame.size.width + PANEL_WIDTH, 0, self.view.frame.size.width, self.view.frame.size.height);
                     }
                     completion:^(BOOL finished) {
                         if (finished) {
                             _mapViewController.rightButton.tag = 0;
                         }
                     }];
    UITapGestureRecognizer *panRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(movePanelToOriginalPosition)];
    
    [_centerViewController.view addGestureRecognizer:panRecognizer];
}

- (void)movePanelToOriginalPosition
{
    [UIView animateWithDuration:SLIDE_TIMING delay:0 options:UIViewAnimationOptionBeginFromCurrentState
                     animations:^{
                         _centerViewController.view.frame = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
                     }
                     completion:^(BOOL finished) {
                         if (finished) {
                             
                             [self resetMainView];
                         }
                     }];
    for (UIGestureRecognizer *recognizer in _centerViewController.view.gestureRecognizers) {
        [_centerViewController.view removeGestureRecognizer:recognizer];
    }
}

- (void)storeFilters:(NSInteger)distance friends:(BOOL)friends fOF:(BOOL)fOF strangers:(BOOL)strangers
{
    self.distance = distance;
    self.friends = friends;
    self.fOF = fOF;
    self.strangers = strangers;
}

#pragma mark -
#pragma mark Default System Code

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

@end
