//
//  MapViewController.m
//  Knock
//
//  Created by Anna Wang on 11/19/15.
//  Copyright © 2015 Anna Wang. All rights reserved.
//

#import "MapViewController.h"
#import "ProfileViewController.h"
#import "ChatViewController.h"
#import "FriendAnnotationView.h"

#define ZOOM_IN .02
#define NORMAL .07
#define ZOOM_OUT .12
#define ME 0
#define FRIEND 1
#define FOF 2
#define STRANGER 3
#define Rgb2UIColor(r, g, b)  [UIColor colorWithRed:((r) / 255.0) green:((g) / 255.0) blue:((b) / 255.0) alpha:1.0]

@interface MapViewController ()

@property (strong, nonatomic) CLLocationManager *manager;
@property (strong, nonatomic) NSArray *closePoints;
@property (strong, nonatomic) NSArray *mediumPoints;
@property (strong, nonatomic) NSArray *farPoints;
@property (strong, nonatomic) MKPointAnnotation *meAnnotation;
@property (strong, nonatomic) CLLocation *meLocation;
@property (weak, nonatomic) IBOutlet UIView *bottomView;
@property (weak, nonatomic) IBOutlet UILabel *bottomView1Label;
@property (strong, nonatomic) NSNumber *count;
@property (weak, nonatomic) IBOutlet UILabel *displayLabel;
@property (weak, nonatomic) IBOutlet UIImageView *bottomViewImage;
@property (weak, nonatomic) IBOutlet UIButton *bottomViewName;
@property (strong, nonatomic) FriendAnnotationView *selectedView;
@property (strong, nonatomic) MKPointAnnotation *selectedAnnotation;
@property (strong, nonatomic) UIImage *selectedAnnotationImage;
@property (strong, nonatomic) NSNumber *zoomLevel;
@property (weak, nonatomic) IBOutlet UIView *legendView;
@property (weak, nonatomic) IBOutlet UIButton *showInfoButton;
@property (weak, nonatomic) IBOutlet UIButton *closeInfoButton;
@property (nonatomic, weak) NSNumber *profileId;
@property (strong, nonatomic) NSString *assoc;
@property (strong, nonatomic) NSString *distanceText;
@property (weak, nonatomic) IBOutlet UIButton *contactBottomLabel;
@end

int count;

@implementation MapViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.friends = YES;
    self.fOF = YES;
    self.strangers = NO;
    self.distance = 2;
    
    self.selectedAnnotation = nil;
    self.mapView.delegate = self;
    self.bottomView.alpha = 0.0f;
    self.legendView.alpha = 0.0f;
    count = 0;
    self.displayLabel.text = @"Friends and Friends of Friends Within 5 Miles";
    _rightButton.tag = 1;
    _manager = [[CLLocationManager alloc] init];
    _manager.delegate = self;
    self.manager.desiredAccuracy = kCLLocationAccuracyBest;
    
    CLLocationDegrees latitude = 52.526732;
    CLLocationDegrees longitude = 13.410542;
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(latitude, longitude);
    //add annotation of location
    self.meAnnotation = [[MKPointAnnotation alloc] init];
    self.meAnnotation.coordinate = coord;
    [self.mapView addAnnotation:self.meAnnotation];
    self.meLocation = [[CLLocation alloc] initWithLatitude:52.526732 longitude:13.410542];
    
    self.zoomLevel = [NSNumber numberWithFloat:ZOOM_IN];
    [self centerMapOnCoordinate:self.meLocation.coordinate withZoom:[self.zoomLevel floatValue]];
    
    [self createAnnotations];
    
    [self updateMap:self.distance friends:self.friends fOF:self.fOF strangers:self.strangers];
    
    self.bottomView1Label.font = [UIFont fontWithName:@"Lato-Light" size:14.f];
    self.bottomViewName.font = [UIFont fontWithName:@"Lato-Light" size:20.f];
    self.contactBottomLabel.font = [UIFont fontWithName:@"Lato-Light" size:14.f];
    [self.bottomViewName setTitleColor:Rgb2UIColor(46, 187, 145) forState:UIControlStateNormal];
    
    self.displayLabel.textColor = Rgb2UIColor(46, 187, 145);
    self.navigationController.navigationBar.tintColor = Rgb2UIColor(46, 187, 145);
    self.navigationController.navigationBar.titleTextAttributes= @{UITextAttributeFont : [UIFont fontWithName:@"Lato-Light" size:22.f]};
    
    if (!UIAccessibilityIsReduceTransparencyEnabled()) {
        self.bottomView.backgroundColor = [UIColor clearColor];
        
        UIBlurEffect *blurEffect = [UIBlurEffect effectWithStyle:UIBlurEffectStyleLight];
        UIVisualEffectView *blurEffectView = [[UIVisualEffectView alloc] initWithEffect:blurEffect];
        blurEffectView.frame = self.view.bounds;
        blurEffectView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        
        [self.bottomView insertSubview:blurEffectView atIndex:0];
    }
    else {
        self.view.backgroundColor = [UIColor blackColor];
    }
}

- (void)centerMapOnCoordinate:(CLLocationCoordinate2D)coordinate withZoom:(CGFloat)zoom
{
    MKCoordinateSpan span = MKCoordinateSpanMake(zoom, zoom);
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(coordinate.latitude, coordinate.longitude);
    MKCoordinateRegion region = MKCoordinateRegionMake(coord, span);
    [self.mapView setRegion:region];
}

- (void)createAnnotations
{
    CLLocation *close1 = [[CLLocation alloc] initWithLatitude:52.522809 longitude:13.413459];
    CLLocation *close2 = [[CLLocation alloc] initWithLatitude:52.522457 longitude:13.402473];
    CLLocation *close3 = [[CLLocation alloc] initWithLatitude:52.527470 longitude:13.412343];
    self.closePoints = [[NSArray alloc] initWithObjects: close1,close2, close3, nil];
    
    CLLocation *med1 = [[CLLocation alloc] initWithLatitude:52.520793 longitude:13.411830];
    CLLocation *med2 = [[CLLocation alloc] initWithLatitude:52.518566 longitude:13.392200];
    CLLocation *med3 = [[CLLocation alloc] initWithLatitude:52.504122 longitude:13.438180];
    self.mediumPoints = [[NSArray alloc] initWithObjects: close1, close2, close3, med1, med2, med3, nil];
    
    CLLocation *far0 = [[CLLocation alloc] initWithLatitude:52.483721 longitude:13.435269];
    CLLocation *far1 = [[CLLocation alloc] initWithLatitude:52.490268 longitude:13.384969];
    CLLocation *far2 = [[CLLocation alloc] initWithLatitude:52.477938 longitude:13.438938];
    CLLocation *far3 = [[CLLocation alloc] initWithLatitude:52.488183 longitude:13.373706];
    CLLocation *far4 = [[CLLocation alloc] initWithLatitude:52.509555 longitude:13.352054];
    CLLocation *far5 = [[CLLocation alloc] initWithLatitude:52.524362 longitude:13.342270];
    CLLocation *far6 = [[CLLocation alloc] initWithLatitude:52.542333 longitude:13.348959];
    
    self.farPoints = [[NSArray alloc] initWithObjects: close1, close2, close3, med1, med2, med3, far0, far1, far2, far3, far4, far5, far6, nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([[segue identifier] isEqualToString:@"profileSegue"])
    {
        ProfileViewController *vc = [segue destinationViewController];
        vc.profileId = self.profileId;
        vc.assocL = self.assoc;
        vc.distanceL = self.distanceText;
    }
    else if ([[segue identifier] isEqualToString:@"bottomBarChatSegue"])
    {
        ChatViewController *vc = [segue destinationViewController];
        vc.profileId = self.profileId;
    }
}


- (IBAction)btnMovePanelLeft:(id)sender
{
    UIBarButtonItem *button = sender;
    switch (button.tag) {
        case 0: {
            [_delegate movePanelToOriginalPosition];
            break;
        }
            
        case 1: {
            [_delegate movePanelLeft];
            break;
        }
            
        default:
            break;
    }
}
- (IBAction)showInfo:(id)sender {
    self.legendView.alpha = 1.0f;
    self.showInfoButton.alpha = 0.0f;
}

- (IBAction)closeInfo:(id)sender {
    self.legendView.alpha = 0.0f;
    self.showInfoButton.alpha = 1.0f;
}

- (void)tappedName
{
    [self performSegueWithIdentifier:@"profileView" sender:self];
    [_delegate movePanelToOriginalPosition];
}

- (void)changeBottomLabel:(NSInteger)distance friends:(BOOL)friends fOF:(BOOL)fOF strangers:(BOOL)strangers
{
    NSString *d = [NSString stringWithFormat: @"%ld", (long)distance];
    NSString *people = [[NSString alloc] init];
    int numSelected = 0;
    if (self.friends){
        numSelected += 1;
    }
    if (self.strangers) {
        numSelected += 5;
    }
    if (self.fOF){
        numSelected += 3;
    }
    if (numSelected == 1){
        people = @"Friends";
    }
    else if (numSelected == 3){
        people = @"Friends of friends";
        
    } else if (numSelected == 5){
        people = @"Strangers";
        
    } else if (numSelected == 4){
        people = @"Friends and friends of friends";
        
    } else if (numSelected == 6){
        people = @"Friends and strangers";
        
    } else if (numSelected == 8){
        people = @"Friends of friends and strangers";
        
    } else if (numSelected == 9){
        people = @"People";
    }
    self.displayLabel.text = [NSString stringWithFormat:@"%@ %@ %@ %@", people, @"within", d, @"miles"];
}

- (void)updateMap:(NSInteger)distance friends:(BOOL)friends fOF:(BOOL)fOF strangers:(BOOL)strangers
{
    self.bottomView.alpha = 0.0f;
    [self.mapView removeAnnotations:self.mapView.annotations];
    [self.mapView addAnnotation:self.meAnnotation];
    
    self.distance = distance;
    self.friends = friends;
    self.fOF = fOF;
    self.strangers = strangers;
    
    [self changeBottomLabel:distance friends:friends fOF:fOF strangers:strangers];
    
    NSArray *possibleAnnotations;
    if (distance == 0){
        self.zoomLevel = [NSNumber numberWithFloat:NORMAL];
        possibleAnnotations = nil;
    } else if (distance > 4 && distance < 7){
        self.zoomLevel = [NSNumber numberWithFloat:NORMAL];
        possibleAnnotations = self.mediumPoints;
        
    } else if (distance >= 7) {
        self.zoomLevel = [NSNumber numberWithFloat:ZOOM_OUT];
        possibleAnnotations = self.farPoints;
    } else {
        self.zoomLevel = [NSNumber numberWithFloat:ZOOM_IN];
        possibleAnnotations = self.closePoints;
    }
    
    CGFloat value = 0;
    if (friends){
        value += .2;
    }
    if (fOF){
        value += .3;
    }
    if (strangers){
        value += .5;
    }
    
    [self centerMapOnCoordinate:self.meLocation.coordinate withZoom:[self.zoomLevel floatValue]];
//    CLLocationDegrees latitude = self.meLocation.coordinate.latitude;
//    CLLocationDegrees longitude = self.meLocation.coordinate.longitude;
//    CLLocationDegrees latDelta = zoom;
//    CLLocationDegrees longDelta = zoom;
//    MKCoordinateSpan span = MKCoordinateSpanMake(latDelta, longDelta);
//    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(latitude, longitude);
//    MKCoordinateRegion region = MKCoordinateRegionMake(coord, span);
//    [self.mapView setRegion:region];
    
    for (int i = 0; i < value * [possibleAnnotations count]; i ++){
        [self.mapView addAnnotation:[possibleAnnotations objectAtIndex:i]];
    }
    
    [self.delegate storeFilters:distance friends:friends fOF:fOF strangers:strangers];

}

- (void)updateFilters:(NSInteger)distance friends:(BOOL)friends fOF:(BOOL)fOF strangers:(BOOL)strangers
{
}


- (IBAction)cancelBottomView:(id)sender {
    
    self.bottomView.alpha = 0.0f;
    self.selectedView.image = self.selectedAnnotationImage;
}

-(FriendAnnotationView *)mapView:(MKMapView *)mV viewForAnnotation:(id <MKAnnotation>)annotation
{
    FriendAnnotationView *pinView = nil;
    if(annotation != self.meAnnotation)
    {
        static NSString *defaultPinID = @"com.invasivecode.pin";
        pinView = (FriendAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[FriendAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];
        int numSelected = 0;
        if (self.friends){
            numSelected += 1;
        }
        if (self.strangers) {
            numSelected += 5;
        }
        if (self.fOF){
            numSelected += 3;
        }
        if (numSelected == 1){
            pinView.image = [UIImage imageNamed:@"friend.png"];
            pinView.type = [NSNumber numberWithInt:FRIEND];
        }
        else if (numSelected == 3){
            pinView.image = [UIImage imageNamed:@"friendsoffriends.png"];
            pinView.type = [NSNumber numberWithInt:FOF];
            
        } else if (numSelected == 5){
            pinView.image = [UIImage imageNamed:@"strangers.png"];
            pinView.type = [NSNumber numberWithInt:STRANGER];
            
        } else if (numSelected == 4){
            if (count %2 == 0){
                pinView.image = [UIImage imageNamed:@"friendsoffriends.png"];
                pinView.type = [NSNumber numberWithInt:FOF];
            } else {
                pinView.image = [UIImage imageNamed:@"friend.png"];
                pinView.type = [NSNumber numberWithInt:FRIEND];
            }
            
        } else if (numSelected == 6){
            if (count %2 == 0){
                pinView.image = [UIImage imageNamed:@"strangers.png"];
                pinView.type = [NSNumber numberWithInt:STRANGER];
            } else {
                pinView.image = [UIImage imageNamed:@"friend.png"];
                pinView.type = [NSNumber numberWithInt:FRIEND];
            }

        } else if (numSelected == 8){
            if (count %2 == 0){
                pinView.image = [UIImage imageNamed:@"strangers.png"];
                pinView.type = [NSNumber numberWithInt:STRANGER];
            } else {
                pinView.image = [UIImage imageNamed:@"friendsoffriends.png"];
                pinView.type = [NSNumber numberWithInt:FOF];
            }
            
        } else if (numSelected == 9){
            if (count % 3== 0 && self.fOF){
                pinView.image = [UIImage imageNamed:@"friendsoffriends.png"];
                pinView.type = [NSNumber numberWithInt:FOF];
            }
            else if (count %3 == 1){
                pinView.image = [UIImage imageNamed:@"strangers.png"];
                pinView.type = [NSNumber numberWithInt:STRANGER];
            } else {
                pinView.image = [UIImage imageNamed:@"friend.png"];
                pinView.type = [NSNumber numberWithInt:FRIEND];
            }
        }
        count++;
        UITapGestureRecognizer *singleFingerTap =
        [[UITapGestureRecognizer alloc] initWithTarget:self
                                                action:@selector(handleSingleTap:)];
        [pinView addGestureRecognizer:singleFingerTap];
    }
    else {
        static NSString *defaultPinID = @"com.invasivecode.pin";
        pinView = (FriendAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[FriendAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];
        pinView.image = [UIImage imageNamed:@"me.png"];
        pinView.type = [NSNumber numberWithInt:ME];
    }
    return pinView;
}

//The event handling method
- (void)handleSingleTap:(UITapGestureRecognizer *)recognizer {
    self.selectedView.image = self.selectedAnnotationImage;
    MKPointAnnotation *annotation = ((MKAnnotationView *)recognizer.view).annotation;
    if (!(annotation.coordinate.longitude == self.meLocation.coordinate.longitude && annotation.
        coordinate.latitude == self.meLocation.coordinate.latitude)){
        if (self.selectedAnnotation.coordinate.latitude == annotation.coordinate.latitude && self.selectedAnnotation.coordinate.longitude == annotation.coordinate.longitude) {
            count--;
        }
        self.selectedAnnotation = annotation;
        [self centerMapOnCoordinate:annotation.coordinate withZoom:[self.zoomLevel floatValue]];
        static NSString *defaultPinID = @"com.invasivecode.pin";
        FriendAnnotationView *pinView = (FriendAnnotationView *)[self.mapView viewForAnnotation:annotation];//(MKAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[FriendAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];
        self.selectedAnnotationImage = pinView.image;
        pinView.image = [UIImage imageNamed:@"chosen.png"];
        self.selectedView = pinView;
        CLLocation *annotationLocation = [[CLLocation alloc] initWithLatitude:annotation.coordinate.latitude longitude:annotation.coordinate.longitude];
        CGFloat distance = [annotationLocation distanceFromLocation:self.meLocation];
        CGFloat miles = 0.000621371192 * distance * 3;
        NSString* formattedNumber = [NSString stringWithFormat:@"%.02f", miles];
        
        NSArray *fofNames = [NSArray arrayWithObjects:@"Felicia Hall", @"Jeff Deere", @"Juliana Bee", @"Stefan Reed", @"Ashley Smith", nil];
        NSArray *fofImages = [NSArray arrayWithObjects:[UIImage imageNamed:@"friend2"],[UIImage imageNamed:@"friend1"], [UIImage imageNamed:@"friend3"],[UIImage imageNamed:@"friend4"], [UIImage imageNamed:@"friend5"],  nil];
        
        NSArray *friendNames = [NSArray arrayWithObjects:@"Sarita Sooklal", @"Dave Fontenot", @"Maia Miller", @"Michael Raitor", @"Isa Badia-Bellinger", nil];
        NSArray *friendImages = [NSArray arrayWithObjects:[UIImage imageNamed:@"friend6"],[UIImage imageNamed:@"friend11"], [UIImage imageNamed:@"friend10"],[UIImage imageNamed:@"friend12"], [UIImage imageNamed:@"friend7"],  nil];
        
        NSArray *strangerNames = [NSArray arrayWithObjects:@"Christine Nguyen", @"Anthony Milki", @"Minna Liu", @"Shankara Anand", @"Nicolas Herrera", nil];
        NSArray *strangerImages = [NSArray arrayWithObjects:[UIImage imageNamed:@"friend8"],[UIImage imageNamed:@"friend15"], [UIImage imageNamed:@"friend9"],[UIImage imageNamed:@"friend14"], [UIImage imageNamed:@"friend13"],  nil];
        
        if ([self.selectedView.type intValue] == FOF){
            int remainder = count % 5;
            self.profileId = [NSNumber numberWithInt: remainder];
            [self.bottomViewName setTitle:[fofNames objectAtIndex:[self.profileId intValue]] forState:UIControlStateNormal];
            [self.bottomViewImage setImage: [fofImages objectAtIndex:[self.profileId intValue]]];
            NSArray *fofAssoc = [NSArray arrayWithObjects:@"Friend of Mark S.", @"Friend of Jason B.", @"Friend of Mark S.", @"Friend of Joanna M.", @"Friend of Alicia L.", nil];
            NSString *primaryText = [fofAssoc objectAtIndex:remainder];
            NSString *text = [NSString stringWithFormat:@"%@ %@ %@", primaryText, formattedNumber, @"miles away"];
            self.bottomView1Label.text = text;
            self.assoc = primaryText;
        } else if ([self.selectedView.type intValue] == FRIEND){
            int remainder = count % 5;
            int prof_id = remainder + 5;
            self.profileId = [NSNumber numberWithInt: prof_id];
            [self.bottomViewName setTitle:[friendNames objectAtIndex:remainder] forState:UIControlStateNormal];
            [self.bottomViewImage setImage: [friendImages objectAtIndex:remainder]];
            NSArray *fofAssoc = [NSArray arrayWithObjects:@"Friend -", @"Friend -", @"Friend -", @"Friend -", @"Friend -", nil];
            NSString *primaryText = [fofAssoc objectAtIndex:remainder];
            NSString *text = [NSString stringWithFormat:@"%@ %@ %@", primaryText, formattedNumber, @"miles away"];
            self.bottomView1Label.text = text;
            self.assoc = @"Friend";
        } else {
            int remainder = count % 5;
            int prof_id = remainder + 10;
            self.profileId = [NSNumber numberWithInt: prof_id];
            [self.bottomViewName setTitle:[strangerNames objectAtIndex:remainder] forState:UIControlStateNormal];
            [self.bottomViewImage setImage: [strangerImages objectAtIndex:remainder]];
            NSArray *fofAssoc = [NSArray arrayWithObjects:@"Stranger -", @"Stranger -", @"Stranger -", @"Stranger -", @"Stranger -", nil];
            NSString *primaryText = [fofAssoc objectAtIndex:remainder];
            NSString *text = [NSString stringWithFormat:@"%@ %@ %@", primaryText, formattedNumber, @"miles away"];
            self.bottomView1Label.text = text;
            self.assoc = @"Stranger";
        }
        self.distanceText = formattedNumber;
        self.bottomView.alpha = 1.0f;
        count ++;
        UITapGestureRecognizer *imageTap =
        [[UITapGestureRecognizer alloc] initWithTarget:self
                                                action:@selector(handleImageTap:)];
        [self.bottomViewImage addGestureRecognizer:imageTap];

    } else {
        self.bottomView.alpha = 0.0f;
    }
    
}

- (void)handleImageTap:(UITapGestureRecognizer *)recognizer {
    [self performSegueWithIdentifier:@"profileSegue" sender:self];
}

@end