//
//  NavigateViewController.m
//  Knock
//
//  Created by Anna Wang on 11/25/15.
//  Copyright © 2015 Anna Wang. All rights reserved.
//

#import "NavigateViewController.h"
#import "KLCPopup.h"

@import Foundation;
#import <QuartzCore/QuartzCore.h>

@interface NavigateViewController () <UITextFieldDelegate, MKMapViewDelegate>

@property (weak, nonatomic) IBOutlet UILabel *bottomLabel;
@property (weak, nonatomic) IBOutlet UINavigationItem *navBar;
@property (weak, nonatomic) IBOutlet UITextField *locationField;
@property (weak, nonatomic) IBOutlet MKMapView *mapView;
@property (weak, nonatomic) IBOutlet UIView *backgroundView;
@property (weak, nonatomic) IBOutlet UIButton *cancelButton;
@property (weak, nonatomic) IBOutlet UIView *friendView;

@property (strong, nonatomic) MKPointAnnotation *meAnnotation;
@property (strong, nonatomic) CLLocation *meLocation;
@property (strong, nonatomic) CLLocation *friendLocation;
@property (strong, nonatomic) MKPointAnnotation *friendAnnotation;

@property (strong, nonatomic) CLLocation *midpointLocation;
@property (strong, nonatomic) MKPointAnnotation *midpointAnnotation;

@property (nonatomic, retain) MKPolyline *routeLine; //your line
@property (nonatomic, retain) MKPolylineView *routeLineView; //overlay view
@property (weak, nonatomic) IBOutlet UIButton *suggestButton;

@property (weak, nonatomic) IBOutlet UIView *calloutView;
@property (weak, nonatomic) IBOutlet UIView *searchView;
@property (weak, nonatomic) IBOutlet UILabel *midpointLabel;
@property (weak, nonatomic) IBOutlet UIView *searchView2;

@end

@implementation NavigateViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.mapView.delegate = self;
    self.locationField.delegate = self;
    self.navBar.title = @"Navigate";
    self.calloutView.alpha = 1.0f;
    self.mapView.userInteractionEnabled = NO;
    self.searchView.alpha = 0.0f;
    self.searchView2.alpha = 0.0f;
    self.friendView.alpha = 0.0f;
    
    //hide everything
    self.cancelButton.alpha = 0.0f;
    self.locationField.alpha = 0.0f;
    self.calloutView.alpha = 0.0f;
    self.backgroundView.alpha = 0.0f;
    self.bottomLabel.text = @"";
    
    UITapGestureRecognizer *tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(editMidpoint)];
    [self.searchView addGestureRecognizer:tapRecognizer];
    UITapGestureRecognizer *tapRecognizer1 = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(editMidpoint1)];
    [self.searchView2 addGestureRecognizer:tapRecognizer1];
    
    CLLocationDegrees latitude = 52.526732;
    CLLocationDegrees longitude = 13.410542;
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(latitude, longitude);
    self.meAnnotation = [[MKPointAnnotation alloc] init];
    self.meAnnotation.coordinate = coord;
    [self.mapView addAnnotation:self.meAnnotation];
    self.meLocation = [[CLLocation alloc] initWithLatitude:52.526732 longitude:13.410542];
    [self centerMapOnCoordinate:self.meLocation.coordinate withZoom:0.01];
    
    self.searchView.alpha = 0.0f;
    self.searchView2.alpha = 0.0f;
    [self.mapView removeOverlays:self.mapView.overlays];
    self.locationField.text = @"";
    
    self.midpointLocation = [[CLLocation alloc] initWithLatitude:52.52383 longitude:13.409308];
    self.midpointAnnotation = [[MKPointAnnotation alloc] init];
    self.midpointAnnotation.coordinate = self.midpointLocation.coordinate;
    [self centerMapOnCoordinate:self.midpointAnnotation.coordinate withZoom:0.01];
    
    self.friendLocation = [[CLLocation alloc] initWithLatitude:52.522800 longitude:13.413459];
    self.friendAnnotation = [[MKPointAnnotation alloc] init];
    self.friendAnnotation.coordinate = self.friendLocation.coordinate;
    [self.mapView addAnnotation:self.friendAnnotation];

//    UIView* contentView = [[UIView alloc] init];
//    contentView.backgroundColor = [UIColor whiteColor];
//    contentView.frame = CGRectMake(0.0, 0.0, 100.0, 100.0);
//    
//    KLCPopup* popup = [KLCPopup popupWithContentView:contentView];
//    [popup show];
    
    [self presentRequestMessage];
    
}
- (IBAction)cancelView:(id)sender {
    self.friendView.alpha = 0.0f;
}

- (void)editMidpoint
{
    self.searchView.alpha = 0.0f;
    self.searchView2.alpha = 0.0f;
    [self.mapView removeOverlays:self.mapView.overlays];
    self.locationField.text = @"";
    [self.mapView removeAnnotation:self.midpointAnnotation];
    
    self.midpointLocation = [[CLLocation alloc] initWithLatitude:52.52403 longitude:13.409308];
    self.midpointAnnotation = [[MKPointAnnotation alloc] init];
    self.midpointAnnotation.coordinate = self.midpointLocation.coordinate;
    
    [self.mapView addAnnotation:self.midpointAnnotation];
    self.midpointLabel.text = @"Dolores";
    self.routeLineView = nil;
    
    CLLocationCoordinate2D first = CLLocationCoordinate2DMake(52.525956, 13.411079);

    CLLocationCoordinate2D sixth = CLLocationCoordinate2DMake(52.523120, 13.413975);
    
    CLLocationCoordinate2D coordinateArray[5];
    coordinateArray[0] = self.meLocation.coordinate;
    coordinateArray[1] = first;
    coordinateArray[2] = self.midpointAnnotation.coordinate;
    coordinateArray[3] = sixth;
    coordinateArray[4] = self.friendAnnotation.coordinate;
    
    self.routeLine = [MKPolyline polylineWithCoordinates:coordinateArray count:5];
    [self.mapView setVisibleMapRect:[self.routeLine boundingMapRect]]; //If you want the route to be visible
    
    [self.mapView addOverlay:self.routeLine];
    [self centerMapOnCoordinate:self.midpointAnnotation.coordinate withZoom:0.01];
}

- (void)editMidpoint1
{
    self.searchView.alpha = 0.0f;
    self.searchView2.alpha = 0.0f;
    [self.mapView removeOverlays:self.mapView.overlays];
    self.locationField.text = @"";
    [self.mapView removeAnnotation:self.midpointAnnotation];
    
    self.midpointLocation = [[CLLocation alloc] initWithLatitude:52.525012 longitude:13.412506];
    self.midpointAnnotation = [[MKPointAnnotation alloc] init];
    self.midpointAnnotation.coordinate = self.midpointLocation.coordinate;
    [self.mapView addAnnotation:self.midpointAnnotation];
    self.midpointLabel.text = @"Escados";
    self.routeLineView = nil;
    
    CLLocationCoordinate2D first = CLLocationCoordinate2DMake(52.525956, 13.411079);
    CLLocationCoordinate2D second = CLLocationCoordinate2DMake(52.525117, 13.413286);
    
    CLLocationCoordinate2D fifth = CLLocationCoordinate2DMake(52.523959, 13.411804);
    CLLocationCoordinate2D sixth = CLLocationCoordinate2DMake(52.523120, 13.413975);
    
    CLLocationCoordinate2D coordinateArray[8];
    coordinateArray[0] = self.meLocation.coordinate;
    coordinateArray[1] = first;
    coordinateArray[2] = second;
    coordinateArray[3] = self.midpointAnnotation.coordinate;
    coordinateArray[4] = fifth;
    coordinateArray[5] = sixth;
    coordinateArray[6] = self.friendAnnotation.coordinate;
    
    self.routeLine = [MKPolyline polylineWithCoordinates:coordinateArray count:7];
    [self.mapView setVisibleMapRect:[self.routeLine boundingMapRect]]; //If you want the route to be visible
    
    [self.mapView addOverlay:self.routeLine];
    [self centerMapOnCoordinate:self.midpointAnnotation.coordinate withZoom:0.01];
}



- (IBAction)suggest:(id)sender {

    UIAlertController * alert =   [UIAlertController
                                  alertControllerWithTitle:@"Location Suggestion Sent!"
                                  message:@""
                                  preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* noButton = [UIAlertAction
                                actionWithTitle:@"Cancel"
                                style:UIAlertActionStyleDefault
                                handler:^(UIAlertAction * action)
                                {
                                    [alert dismissViewControllerAnimated:YES completion:nil];
                                }];
    UIAlertAction* yesButton = [UIAlertAction
                               actionWithTitle:@"OK"
                               style:UIAlertActionStyleDefault
                               handler:^(UIAlertAction * action)
                               {
                                   self.bottomLabel.text = [NSString stringWithFormat:@"%@ %@ %@", @"Location Suggestion Sent: ", self.midpointLabel.text, @"Café"];
                                   [alert dismissViewControllerAnimated:YES completion:nil];
                                   self.suggestButton.alpha = 0.0f;
                                   double delayInSeconds = 1.0;
                                   [[UIApplication sharedApplication] beginIgnoringInteractionEvents];
                                   dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
                                   dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
                                       //code to be executed on the main queue after delay
                                       [[UIApplication sharedApplication] endIgnoringInteractionEvents];
                                       [self presentAcceptMessage];
                                   });
                               }];
    
    [alert addAction:noButton];
    [alert addAction:yesButton];
    
    [self presentViewController:alert animated:YES completion:nil];
    
}

- (void)presentRequestMessage
{
    UIAlertController * alert =   [UIAlertController
                                   alertControllerWithTitle:@"Meet Request Sent!"
                                   message:@""
                                   preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* noButton = [UIAlertAction
                               actionWithTitle:@"Cancel"
                               style:UIAlertActionStyleDefault
                               handler:^(UIAlertAction * action)
                               {
                                   [alert dismissViewControllerAnimated:YES completion:nil];
                                   [self performSegueWithIdentifier:@"backToMessages" sender:self];
                               }];
    UIAlertAction* yesButton = [UIAlertAction
                                actionWithTitle:@"OK"
                                style:UIAlertActionStyleDefault
                                handler:^(UIAlertAction * action)
                                {
                                    self.bottomLabel.text = @"Meet Request Sent";
                                    [alert dismissViewControllerAnimated:YES completion:nil];
                                    double delayInSeconds = 1.0;
                                    [[UIApplication sharedApplication] beginIgnoringInteractionEvents];
                                    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
                                    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
                                        //code to be executed on the main queue after delay
                                        [self presentRequestAcceptMessage];
                                        [[UIApplication sharedApplication] endIgnoringInteractionEvents];
                                    });
                                    
                                }];
    
    [alert addAction:noButton];
    [alert addAction:yesButton];
    
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)presentRequestAcceptMessage
{
    UIAlertController * alert =   [UIAlertController
                                   alertControllerWithTitle:@"Meet Request Accepted!"
                                   message:@""
                                   preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* yesButton = [UIAlertAction
                                actionWithTitle:@"OK"
                                style:UIAlertActionStyleDefault
                                handler:^(UIAlertAction * action)
                                {
                                    //Handle your yes please button action here
                                    self.bottomLabel.text = @"Meet Request Accepted";
                                    [alert dismissViewControllerAnimated:YES completion:nil];
                                    [self editMidpoint1];
                                    self.locationField.alpha = 1.0f;
                                    self.cancelButton.alpha = 1.0f;
                                    self.calloutView.alpha = 1.0f;
                                    self.backgroundView.alpha = 1.0f;
                                    
                                }];
//    UIAlertAction* noButton = [UIAlertAction
//                               actionWithTitle:@"Open in Maps"
//                               style:UIAlertActionStyleDefault
//                               handler:^(UIAlertAction * action)
//                               {
//                                   [alert dismissViewControllerAnimated:YES completion:nil];
//                                   double delayInSeconds = 1.0;
//                                   dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
//                                   dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
//                                       //code to be executed on the main queue after delay
//                                       [self presentGoogleMapsMessage];
//                                   });
//                                   
//                                   
//                               }];
    
    [alert addAction:yesButton];
    
    [self presentViewController:alert animated:YES completion:nil];
    
}

- (void)presentAcceptMessage
{
    UIAlertController * alert =   [UIAlertController
                                   alertControllerWithTitle:@"Location Suggestion Accepted!"
                                   message:@""
                                   preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* yesButton = [UIAlertAction
                                actionWithTitle:@"OK"
                                style:UIAlertActionStyleDefault
                                handler:^(UIAlertAction * action)
                                {
                                    //Handle your yes please button action here
                                    self.bottomLabel.text = [NSString stringWithFormat:@"%@ %@ %@", @"Location Suggestion Accepted: ", self.midpointLabel.text, @"Café"];
                                    [alert dismissViewControllerAnimated:YES completion:nil];
                                    
                                }];
    UIAlertAction* noButton = [UIAlertAction
                               actionWithTitle:@"Open in Maps"
                               style:UIAlertActionStyleDefault
                               handler:^(UIAlertAction * action)
                               {
                                   [alert dismissViewControllerAnimated:YES completion:nil];
                                   double delayInSeconds = 1.0;
                                   [[UIApplication sharedApplication] beginIgnoringInteractionEvents];
                                   dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
                                   dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
                                       //code to be executed on the main queue after delay
                                       [self presentGoogleMapsMessage];
                                       [[UIApplication sharedApplication] endIgnoringInteractionEvents];
                                   });
                                                                      self.bottomLabel.text = [NSString stringWithFormat:@"%@ %@ %@", @"Location Suggestion Accepted: ", self.midpointLabel.text, @"Café"];
                                   
                               }];
    
    [alert addAction:yesButton];
    [alert addAction:noButton];
    
    [self presentViewController:alert animated:YES completion:nil];

}

- (void)presentGoogleMapsMessage
{
    UIAlertController * alert =   [UIAlertController
                                   alertControllerWithTitle:@"Open in Google Maps?"
                                   message:@""
                                   preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* yesButton = [UIAlertAction
                                actionWithTitle:@"OK"
                                style:UIAlertActionStyleDefault
                                handler:^(UIAlertAction * action)
                                {
                                    //Handle your yes please button action here
                                    [alert dismissViewControllerAnimated:YES completion:nil];
                                }];
    UIAlertAction* noButton = [UIAlertAction
                               actionWithTitle:@"Don't Allow"
                               style:UIAlertActionStyleDefault
                               handler:^(UIAlertAction * action)
                               {
                                   [alert dismissViewControllerAnimated:YES completion:nil];
                               }];
    
    [alert addAction:noButton];
    [alert addAction:yesButton];
    
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)openAnnotation:(id)annotation;
{
    //mv is the mapView
    [self.mapView selectAnnotation:annotation animated:YES];
    
}

-(MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id<MKOverlay>)overlay
{
    if(overlay == self.routeLine)
    {
        if(nil == self.routeLineView)
        {
            self.routeLineView = [[MKPolylineView alloc] initWithPolyline:self.routeLine];
            self.routeLineView.fillColor = [UIColor blueColor];
            self.routeLineView.strokeColor = [UIColor blueColor];
            self.routeLineView.lineWidth = 8;
            
        }
        
        return self.routeLineView;
    }
    
    return nil;
}

- (void)centerMapOnCoordinate:(CLLocationCoordinate2D)coordinate withZoom:(CGFloat)zoom
{
    MKCoordinateSpan span = MKCoordinateSpanMake(zoom, zoom);
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(coordinate.latitude, coordinate.longitude);
    MKCoordinateRegion region = MKCoordinateRegionMake(coord, span);
    [self.mapView setRegion:region];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(BOOL) textFieldShouldReturn:(UITextField *)textField{
    
    [textField resignFirstResponder];
    if ([[textField.text lowercaseString] isEqualToString: @"food"]){
        self.searchView.alpha = 1.0f;
        self.searchView2.alpha = 1.0f;
    } else {
        self.searchView.alpha = 0.0f;
        self.searchView2.alpha = 0.0f;
    }
    
    return YES;
}

- (IBAction)cancelSearch:(id)sender {
    self.searchView.alpha = 0.0f;
    self.searchView2.alpha = 0.0f;
    self.locationField.text = @"";
}

-(MKAnnotationView *)mapView:(MKMapView *)mV viewForAnnotation:(id <MKAnnotation>)annotation
{
    MKAnnotationView *pinView = nil;
    if(annotation == self.meAnnotation)
    {
        static NSString *defaultPinID = @"com.invasivecode.pin";
        pinView = (MKAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[MKAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];        pinView.image = [UIImage imageNamed:@"me.png"];
    }
    else if (annotation == self.friendAnnotation){
        static NSString *defaultPinID = @"com.invasivecode.pin";
        pinView = (MKAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[MKAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];
        pinView.image = [UIImage imageNamed:@"friendsoffriends.png"];
        UITapGestureRecognizer *singleFingerTap =
        [[UITapGestureRecognizer alloc] initWithTarget:self
                                                action:@selector(handleSingleTap:)];
        [pinView addGestureRecognizer:singleFingerTap];
    } else {
        static NSString *defaultPinID = @"com.invasivecode.pin";
        pinView = (MKAnnotationView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:defaultPinID];
        if ( pinView == nil )
            pinView = [[MKAnnotationView alloc]
                       initWithAnnotation:annotation reuseIdentifier:defaultPinID];
        pinView.image = [UIImage imageNamed:@"pin.png"];
        pinView.tintColor = [UIColor blueColor];

    }
    
    return pinView;
}

- (void)handleSingleTap:(UITapGestureRecognizer *)recognizer {
    self.friendView.alpha = 1.0f;
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
